////////////////////////////////////////////////////////////////////////
//
// 46-927 Data Structures, Algorithms and AI
//
// File: EuropeanCallOption.cpp
//
// Author:        Jeff Stephenson        
// Date:          September 17, 1998  
// Comments:      Implementation of the CEuropeanCallOption class
//
//                A class derived from CAbstractOption for
//                pricing European Call Options
//
//                See EuropeanCallOption.h
//
//                Importantly, we are defining the F() and Payoff()
//                functions for a European Call Option.  Recall that
//                for a European Call Option the auxiliary process is
//                not needed, therefore the stock price is used.  The 
//                payoff is simply the price of the stock minus the 
//                strike price.
//
// 
////////////////////////////////////////////////////////////////////////


#include <iostream.h>
#include "EuropeanCallOption.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEuropeanCallOption::CEuropeanCallOption(int T, 
                                         double r, 
                                         double u, 
                                         double d, 
                                         double s_0):CAbstractOption(T, r, u, d, s_0) 
{
    //work handled by base class
    
}


CEuropeanCallOption::CEuropeanCallOption(int T, 
                                         double r, 
                                         double u, 
                                         double d, 
                                         double s_0,
                                         double strike):CAbstractOption(T, r, u, d, s_0) 
{
    //work handled by base class
    m_strike_price = strike;
}

CEuropeanCallOption::~CEuropeanCallOption()
{

}

void CEuropeanCallOption::SetStrikePrice(double strike)
{
    m_strike_price = strike;
}

double CEuropeanCallOption::F(double s, double x)
{
    return s;
}

double CEuropeanCallOption::Payoff(double s, double x, int t)
{
    double value = s-m_strike_price;
    if (value < 0) value=0.0;
    return(value);
}
