////////////////////////////////////////////////////////////////////////
//
// 46-927 Data Structures, Algorithms and AI
//
// File: EuropeanCallOption.h
//
// Author:        Jeff Stephenson        
// Date:          September 17, 1998  
// Comments:      Declaration of the CEuropeanCallOption class
//
//                A class derived from CAbstractOption for
//                pricing European Call Options.
//
//                We have added a member variable for the strike price 
//                (m_strike_price).  In addition we have declared 
//                the F() and Payoff() functions.
//
//                See EuropeanCallOption.cpp for more detail.
// 
////////////////////////////////////////////////////////////////////////


#ifndef EUROPEANCALLOPT_H
#define EUROPEANCALLOPT_H

#include "AbstractOption.h"

class CEuropeanCallOption : public CAbstractOption  
{
public:
	CEuropeanCallOption(int T, 
                        double r, 
                        double u, 
                        double d, 
                        double s_0);

    CEuropeanCallOption(int T, 
                        double r, 
                        double u, 
                        double d, 
                        double s_0,
                        double strike);

	virtual ~CEuropeanCallOption();

    void SetStrikePrice(double strike);
    double F(double s, double x);
    double Payoff(double s, double x, int t);

private:

    double m_strike_price;

};

#endif 