////////////////////////////////////////////////////////////////////////
//
// 46-927 Data Structures, Algorithms and AI
//
// File: LinkedList.cpp
//
// Author:        Jeff Stephenson         
// Date:          August 22, 1998 (modified 9/17/98)
// Comments:      Implementation of the CLinkedList class
//
//                9/17/98 -- Changed the Insertion routines
//                           to return a CNode pointer to the
//                           node just inserted (instead of void).
//
// 
////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include "LinkedList.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLinkedList::CLinkedList()
{
    m_head = NULL;
    m_tail = NULL;
    m_count = 0;
}

CLinkedList::~CLinkedList()
{
    CNode* temp;
    while (m_head != NULL)
    {
        temp = m_head;
        m_head = m_head->GetNext();
        delete temp;
    }
}

//////////////////////////////////////////////////////////////////////
// Insertion Routines
//////////////////////////////////////////////////////////////////////
CNode* CLinkedList::InsertAtHead(CKey aKey)
{
    CNode *newNode;
    newNode = new CNode(aKey);

    if (m_head == NULL)
    {   
        m_head = newNode;
        m_tail = newNode;
    }
    else
    {
        newNode->SetNext(m_head);
        m_head->SetPrev(newNode);
        m_head = newNode;
    }

    m_count = m_count + 1;
    return newNode;
}

CNode* CLinkedList::InsertAtTail(CKey aKey)
{
    CNode* newNode;
    newNode = new CNode(aKey);

    if (m_head == NULL)
    {
        m_head = newNode;
        m_tail = newNode;
    }
    else
    {
        newNode->SetPrev(m_tail);
        m_tail->SetNext(newNode);
        m_tail = newNode;
    }

    m_count = m_count + 1;
    return newNode;
}


CNode* CLinkedList::InsertUnique(CKey aKey)
{
    CNode* found;

    found = Search(aKey);

    if (found == NULL)
    {
        found = InsertAtTail(aKey);
    }

    return found;
}

//////////////////////////////////////////////////////////////////////
// Counts the number of nodes in the linked list
// Returns:  The number of nodes
//////////////////////////////////////////////////////////////////////
int  CLinkedList::CountNodes()
{
    return m_count;
}


//////////////////////////////////////////////////////////////////////
// Searches the linked list for aKey
// Returns:  Returns node if found, NULL otherwise
//////////////////////////////////////////////////////////////////////
CNode* CLinkedList::Search(CKey aKey)
{
    CNode* temp;
    CKey   tempData;

    temp = m_head;

    while (temp != NULL)
    {
        tempData = temp->GetData();
        if (tempData.IsEqual(aKey))
        {
            break;
        }
        temp = temp->GetNext();
    }

    return temp;
}

//////////////////////////////////////////////////////////////////////
// Printing Function, prints nodes from head down
//////////////////////////////////////////////////////////////////////
void CLinkedList::PrintList()
{
    CNode* temp;

    temp = m_head;

    if (temp == NULL)
    {
        cout << "List is Empty" << endl;
        return;
    }

    while (temp != NULL)
    {
        temp->PrintNode();
        temp = temp->GetNext();
    }
}


bool CLinkedList::RemoveFromHead(CKey& retVal)
{
    CNode* temp;
    bool   success = false;

    if (m_head != NULL)
    {

        temp = m_head;
        m_head = m_head->GetNext();
        m_head->SetPrev(NULL);
        retVal = temp->GetData();
        m_count = m_count - 1;

        success = true;
        delete temp;  //free CNode memory
    }

    if (m_count == 0)
    {
        m_tail = NULL;
    }

    return success;

}
